const mqtt = require('mqtt');
var moment = require('moment');

const { HOST, PORT, TOKEN, INTERVAL, PUB_TOPIC } = require('./config.js');

// MQTT broker url
const URL = `mqtt://${HOST}:${PORT}`

// Counters for statistics
let globalCount = 0;
let globalCountLBS = 0;
let count = 0;
let countLBS = 0;

/** Connect to MQTT broker
 */
const client  = mqtt.connect(URL, {
    username: TOKEN,
    clientId: 'nodejs'
});

/** Handle 'connect' event
 */
client.on('connect', function () {
    console.log('Connected to', URL);

    // subscribe topic
    client.subscribe('debug_stream/#')
})

/** Handle 'error' event
 */
client.on('disconnect', function () {
    console.log('error', arguments);
})

/** Handle 'error' event
 */
client.on('error', function (evt) {
    console.log('error', arguments);
})

/** Handle 'connect' event
 */
client.on('message', function (topic, message) {
    // message is Buffer
    let data = JSON.parse(message.toString()) || {};

    // increment received messages counter
    count++;

    // Process messages
    if (data.id) {
        let params = data.msg.p;
        let pos = data.msg.pos;

        // check message for LBS params
        if (params && pos && 'mnc' in params && 'mcc' in params && 'lac' in params && 'cell_id' in params) {
            // LBS message found, increment counter
            countLBS++;
            // publish LBS message to specific topic
            client.publish(PUB_TOPIC, JSON.stringify({pos: pos, params: params}))
        }
    }
})

// Log statistics (every interval ms)
const interval = INTERVAL | 1000;

setInterval(() => {
    // summ counters
    globalCount += count;
    globalCountLBS += countLBS;

    // log stats
    console.log(moment().format('YYYY/MM/DD HH:mm:ss'), (count / (interval / 1000)).toFixed(0), 'msgs/sec, lbs:', (countLBS / (interval / 1000)).toFixed(0), 'msgs/sec');

    // reset counters
    count = 0;
    countLBS = 0;
}, interval);
